# -*- coding: utf-8 -*-
#
# Copyright 2011-2012 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The base widget for the Control Panel tabs."""

from PyQt4 import QtGui, QtCore

from twisted.internet import defer
from ubuntu_sso.utils.ui import CLOSE_AND_SETUP_LATER

from ubuntuone.controlpanel import cache
from ubuntuone.controlpanel.gui.qt.signin import SignInPanel
from ubuntuone.controlpanel.gui.qt.side_widget import SideWidget


class UbuntuOnePage(QtGui.QWizardPage):
    """A generic page for the UbuntuOneWizard."""

    panel_class = None

    def __init__(self, *args, **kwargs):
        super(UbuntuOnePage, self).__init__(*args, **kwargs)

        self.layout = QtGui.QVBoxLayout(self)
        self.panel = None
        if self.panel_class is not None:
            self.panel = SignInPanel()
            self.layout.addWidget(self.panel)


class SignInPage(UbuntuOnePage):
    """The page to signin to Ubuntu One."""

    panel_class = SignInPanel


class UbuntuOneWizard(cache.Cache, QtGui.QWizard):
    """The Ubuntu One wizard."""

    def __init__(self, *args, **kwargs):
        super(UbuntuOneWizard, self).__init__(*args, **kwargs)
        self.pages = {}

        self.setOption(self.NoBackButtonOnStartPage, True)
        self.setOption(self.HaveFinishButtonOnEarlyPages, False)
        self.setWizardStyle(self.ModernStyle)

        self.setButtonText(self.CancelButton, CLOSE_AND_SETUP_LATER)
        self.setButtonLayout([self.Stretch, self.CancelButton])

        self.side_widget = SideWidget()
        self.side_widget.stage = self.side_widget.signin_stage
        self.setSideWidget(self.side_widget)

        self.signin_page = SignInPage()
        self.addPage(self.signin_page)

        self.signin_page.panel.ui.login_button.clicked.connect(self.login)
        self.signin_page.panel.ui.register_button.clicked.connect(
            self.register)

        self.setTabOrder(self.signin_page.panel.ui.login_button,
                         self.signin_page.panel.ui.register_button)
        self.setTabOrder(self.signin_page.panel.ui.register_button,
                         self.button(self.CancelButton))

    # pylint: disable=C0103

    def addPage(self, page):
        """Add 'page' to this wizard, and keep a reference of its ID."""
        page_id = super(UbuntuOneWizard, self).addPage(page)
        self.pages[page] = page_id

    # pylint: enable=C0103

    @QtCore.pyqtSlot()
    @defer.inlineCallbacks
    def login(self):
        """Show the login dialog."""
        self.setEnabled(False)
        credentials = yield self.backend.login()
        self._process_credentials(credentials)
        self.setEnabled(True)

    @QtCore.pyqtSlot()
    @defer.inlineCallbacks
    def register(self):
        """Show the register dialog."""
        self.setEnabled(False)
        credentials = yield self.backend.register()
        self._process_credentials(credentials)
        self.setEnabled(True)

    def _process_credentials(self, credentials=None):
        """Confirm which is the next step after analyzing 'credentials'."""
        if credentials:
            self.accept()
