# -*- coding: utf-8 -*-

#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Autoupdate functions for windows."""

import os
import sys

# Avoid pylint error on Linux
# pylint: disable=F0401
import win32api
# pylint: enable=F0401

from twisted.internet import defer
from twisted.internet.utils import getProcessValue

from ubuntuone.controlpanel.logger import setup_logging

logger = setup_logging('utils')
AUTOUPDATE_EXE_NAME = 'autoupdate-windows.exe'


def _get_update_path():
    """Return the path in which the autoupdate command is found."""
    if hasattr(sys, 'frozen'):
        exec_path = os.path.abspath(sys.executable)
    else:
        exec_path = os.path.dirname(__file__)
    folder = os.path.dirname(exec_path)
    update_path = os.path.join(folder, AUTOUPDATE_EXE_NAME)
    if os.path.exists(update_path):
        return update_path
    return None


@defer.inlineCallbacks
def are_updates_present():
    """Return if there are updates for Ubuntu One."""
    result = False
    retcode = None
    update_path = _get_update_path()
    if update_path is not None:
        # If there is an update present we will get 0, non-zero otherwise
        retcode = yield getProcessValue(update_path, args=('--mode',
            'unattended'), path=os.path.dirname(update_path))
        result = retcode == 0
    logger.debug('are_updates_present: update path: %r, return code: %r,'
                 ' result: %r', update_path, retcode, result)
    defer.returnValue(result)


def perform_update():
    """Spawn the autoupdate process and call the stop function."""
    update_path = _get_update_path()
    if update_path is not None:
        # lets call the updater with the commands that are required,
        win32api.ShellExecute(None, 'runas',
            update_path,
            '--unattendedmodeui none', '', 0)
